@@ -2,7 +2,7 @@
  *
  *  Wireless daemon for Linux
  *
- *  Copyright (C) 2013-2014  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2013-2018  Intel Corporation. All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
@@ -45,6 +45,7 @@ enum crypto_akm {
 	CRYPTO_AKM_8021X_SUITE_B_SHA256 = 0x000fac0b,
 	CRYPTO_AKM_8021X_SUITE_B_SHA384 = 0x000fac0c,
 	CRYPTO_AKM_FT_OVER_8021X_SHA384 = 0x000fac0d,
+	CRYPTO_AKM_OWE = 0x000fac12,
 };
 
 /* Min & Max reported by crypto_cipher_key_len when ignoring WEP */
@@ -53,12 +54,6 @@ enum crypto_akm {
 #define CRYPTO_MIN_IGTK_LEN 16
 #define CRYPTO_MAX_IGTK_LEN 32
 
-struct crypto_ptk {
-	uint8_t kck[16];
-	uint8_t kek[16];
-	uint8_t tk[0];
-} __attribute__ ((packed));
-
 extern const unsigned char crypto_dh5_prime[];
 extern size_t crypto_dh5_prime_size;
 extern const unsigned char crypto_dh5_generator[];
@@ -70,10 +65,12 @@ bool hmac_sha1(const void *key, size_t key_len,
 		const void *data, size_t data_len, void *output, size_t size);
 bool hmac_sha256(const void *key, size_t key_len,
 		const void *data, size_t data_len, void *output, size_t size);
+bool hmac_sha384(const void *key, size_t key_len,
+		const void *data, size_t data_len, void *output, size_t size);
 bool cmac_aes(const void *key, size_t key_len,
 		const void *data, size_t data_len, void *output, size_t size);
 
-bool aes_unwrap(const uint8_t *kek, const uint8_t *in, size_t len,
+bool aes_unwrap(const uint8_t *kek, size_t kek_len, const uint8_t *in, size_t len,
 			uint8_t *out);
 bool aes_wrap(const uint8_t *kek, const uint8_t *in, size_t len, uint8_t *out);
 bool arc4_skip(const uint8_t *key, size_t key_len, size_t skip,
@@ -92,11 +89,17 @@ bool kdf_sha256(const void *key, size_t key_len,
 bool prf_sha1(const void *key, size_t key_len,
 		const void *prefix, size_t prefix_len,
 		const void *data, size_t data_len, void *output, size_t size);
+bool hkdf_extract(enum l_checksum_type type, const uint8_t *key, size_t key_len,
+				uint8_t num_args, uint8_t *out, ...);
 
-bool crypto_derive_pairwise_ptk(const uint8_t *pmk,
+bool hkdf_expand(enum l_checksum_type type, const uint8_t *key, size_t key_len,
+				const char *info, size_t info_len, void *out,
+				size_t out_len);
+
+bool crypto_derive_pairwise_ptk(const uint8_t *pmk, size_t pmk_len,
 				const uint8_t *addr1, const uint8_t *addr2,
 				const uint8_t *nonce1, const uint8_t *nonce2,
-				struct crypto_ptk *out_ptk, size_t ptk_len,
+				uint8_t *out_ptk, size_t ptk_len,
 				bool use_sha256);
 
 bool crypto_derive_pmk_r0(const uint8_t *xxkey,
@@ -113,7 +116,7 @@ bool crypto_derive_pmk_r1(const uint8_t *pmk_r0,
 bool crypto_derive_ft_ptk(const uint8_t *pmk_r1, const uint8_t *pmk_r1_name,
 				const uint8_t *addr1, const uint8_t *addr2,
 				const uint8_t *nonce1, const uint8_t *nonce2,
-				struct crypto_ptk *out_ptk, size_t ptk_len,
+				uint8_t *out_ptk, size_t ptk_len,
 				uint8_t *out_ptk_name);
 
 bool crypto_derive_pmkid(const uint8_t *pmk,
