@@ -44,6 +44,8 @@
 #include "src/plugin.h"
 #include "src/simauth.h"
 #include "src/adhoc.h"
+#include "src/blacklist.h"
+#include "src/storage.h"
 
 #include "src/backtrace.h"
 
@@ -445,7 +447,7 @@ int main(int argc, char *argv[])
 	if (debugopt)
 		l_debug_enable(debugopt);
 
-#ifdef HAVE_EXECINFO_H
+#ifdef HAVE_BACKTRACE
 	__iwd_backtrace_init();
 #endif
 
@@ -469,6 +471,11 @@ int main(int argc, char *argv[])
 
 	exit_status = EXIT_FAILURE;
 
+	if (create_dirs(DAEMON_STORAGEDIR "/")) {
+		l_error("Failed to create " DAEMON_STORAGEDIR "/");
+		goto fail_dbus;
+	}
+
 	dbus = l_dbus_new_default(L_DBUS_SYSTEM_BUS);
 	if (!dbus) {
 		l_error("Failed to initialize D-Bus");
@@ -498,9 +505,11 @@ int main(int argc, char *argv[])
 	known_networks_init();
 	sim_auth_init();
 	plugin_init(plugins, noplugins);
+	blacklist_init();
 
 	exit_status = l_main_run_with_signal(signal_handler, NULL);
 
+	blacklist_exit();
 	plugin_exit();
 	sim_auth_exit();
 	known_networks_exit();
